<?php

namespace Joolano\Joolano\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Joolano\Joolano\Http\Requests\ServiceTypeRequest;
use Joolano\Joolano\Interfaces\ServiceTypeRepositoryInterface;
use Joolano\Joolano\Models\ServiceType;

/**
 * Resource controller class for service_type.
 */
class ServiceTypeResourceController extends BaseController
{

    /**
     * Initialize service_type resource controller.
     *
     * @param type ServiceTypeRepositoryInterface $service_type
     *
     * @return null
     */
    public function __construct(ServiceTypeRepositoryInterface $service_type)
    {
        parent::__construct();
        $this->repository = $service_type;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Joolano\Joolano\Repositories\Criteria\ServiceTypeResourceCriteria::class);
    }

    /**
     * Display a list of service_type.
     *
     * @return Response
     */
    public function index(ServiceTypeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Joolano\Joolano\Repositories\Presenter\ServiceTypePresenter::class)
                ->$function();
        }

        $service_types = $this->repository->paginate();

        return $this->response->title(trans('joolano::service_type.names'))
            ->view('joolano::service_type.index', true)
            ->data(compact('service_types', 'view'))
            ->output();
    }

    /**
     * Display service_type.
     *
     * @param Request $request
     * @param Model   $service_type
     *
     * @return Response
     */
    public function show(ServiceTypeRequest $request, ServiceType $service_type)
    {

        if ($service_type->exists) {
            $view = 'joolano::service_type.show';
        } else {
            $view = 'joolano::service_type.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('joolano::service_type.name'))
            ->data(compact('service_type'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new service_type.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ServiceTypeRequest $request)
    {

        $service_type = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('joolano::service_type.name')) 
            ->view('joolano::service_type.create', true) 
            ->data(compact('service_type'))
            ->output();
    }

    /**
     * Create new service_type.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ServiceTypeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $service_type                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('joolano::service_type.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('joolano/service_type/' . $service_type->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/joolano/service_type'))
                ->redirect();
        }

    }

    /**
     * Show service_type for editing.
     *
     * @param Request $request
     * @param Model   $service_type
     *
     * @return Response
     */
    public function edit(ServiceTypeRequest $request, ServiceType $service_type)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('joolano::service_type.name'))
            ->view('joolano::service_type.edit', true)
            ->data(compact('service_type'))
            ->output();
    }

    /**
     * Update the service_type.
     *
     * @param Request $request
     * @param Model   $service_type
     *
     * @return Response
     */
    public function update(ServiceTypeRequest $request, ServiceType $service_type)
    {
        try {
            $attributes = $request->all();

            $service_type->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('joolano::service_type.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('joolano/service_type/' . $service_type->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('joolano/service_type/' . $service_type->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the service_type.
     *
     * @param Model   $service_type
     *
     * @return Response
     */
    public function destroy(ServiceTypeRequest $request, ServiceType $service_type)
    {
        try {

            $service_type->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('joolano::service_type.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('joolano/service_type/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('joolano/service_type/' . $service_type->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple service_type.
     *
     * @param Model   $service_type
     *
     * @return Response
     */
    public function delete(ServiceTypeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('joolano::service_type.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('joolano/service_type'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/joolano/service_type'))
                ->redirect();
        }

    }

    /**
     * Restore deleted service_types.
     *
     * @param Model   $service_type
     *
     * @return Response
     */
    public function restore(ServiceTypeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('joolano::service_type.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/joolano/service_type'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/joolano/service_type/'))
                ->redirect();
        }

    }

}
